#include <bits/stdc++.h>
using namespace std;

struct Node{
    int key;
    int sz;
    Node* left;
    Node* right;
    Node(int k){key=k; sz=1; left=0; right=0;}
};

Node* insertNode(Node* node, int k);
Node* rotateLeft(Node* node);
Node* rotateRight(Node* node);
Node* insertRoot(Node* node, int k);
int getSize(Node* node);
void fixSize(Node* node);
Node* findNode(Node* node, int key);
Node* removeNode(Node* node);
int getMax(Node* node);

Node* insertNode(Node* node, int k){
    if(!node)
        return new Node(k);
    if(rand()%(node->sz+1) == 0)
        return insertRoot(node, k);
    if(node->key > k){
        node->left = insertNode(node->left, k);
    } else {
        node->right = insertNode(node->right, k);
    }
}

Node* rotateLeft(Node* node){
    Node* newRoot = node->right;
    if(!newRoot) return node;
    node->right = newRoot->left;
    newRoot->left = node;
    newRoot->sz = node->sz;
    fixSize(node);
    return newRoot;
}

Node* rotateRight(Node* node){
    Node* newRoot = node->left;
    if(!newRoot) return node;
    node->left = newRoot->right;
    newRoot->right = node;
    fixSize(node);
    fixSize(newRoot);
    return newRoot;
}

Node* insertRoot(Node* node, int k){
    if(!node)
        return new Node(k);
    if(k < node->key){
       node->left = insertRoot(node->left, k);
       return rotateRight(node);
    } else {
        node->right = insertRoot(node->right, k);
        return rotateLeft(node);
    }
}

int getSize(Node* node){
    if(!node)
        return 0;
    return node->sz;
}

void fixSize(Node* node){
    node->sz = 1 + getSize(node->left) + getSize(node->right);
}

Node* findNode(Node* node, int key){
    if(!node)
        return 0;
    if(node->key == key)
        return node;
    if(key < node->key)
        return findNode(node->left, key);
    else
        return findNode(node->right, key);
}

Node* removeNode(Node* parent, int key){
    if(parent->key != key){
        if(key < parent->key){
            parent->left = removeNode(parent->left, key);
        } else {
            parent->right = removeNode(parent->right, key);
        }
        return parent;
    } else {
        if(!parent->left)
            return parent->right;
        if(!parent->right)
            return parent->left;
        Node* newNode;
        if(!rand()%2){
            newNode = rotateLeft(parent);
            newNode->left = removeNode(newNode->left, key);
        } else {
            newNode = rotateRight(parent);
            newNode->right = removeNode(newNode->right, key);
        }
        return newNode;
    }
}

int getMax(Node* node){
    if(!node)
        return 0;
    while(node->right)
        node = node->right;
    return node->key;
}

void eraseTree(Node* node){
    if(!node)
        return;
    eraseTree(node->left);
    eraseTree(node->right);
    delete(node);
}

void input();
void getDataBack();

const bool DEBUG = false;

const int SIZE = 1000;
int data[SIZE][SIZE];
int dataCP[SIZE][SIZE];
int precount[SIZE];
int n;

int getAnswer(int sz1, int sz2){
    int minn = n*n;
    for(int i=0; i<sz1; i++)
        for(int j=0; j<sz2; j++)
            minn = min(minn, data[i][j]);
    return minn;
}

void solveVertical(int sz, int b){
    Node* bst = 0;
    int newNum;
    for(int j=0; j<sz; j++){
        for(int i=0; i<b-1; i++) //insert b-1 from row j
            bst = insertNode(bst, data[i][j]);
        for(int i=b-1, k=0; i<n; i++, k++){
            bst = insertNode(bst, data[i][j]);
            newNum = getMax(bst);
            bst = removeNode(bst, data[k][j]);
            data[k][j] = newNum;
        }
        eraseTree(bst);
        bst = 0;
    }
    if(DEBUG){
        for(int i=0; i<n-b+1; i++){
            for(int j=0; j<sz; j++){
                cout<<data[i][j]<<' ';
            }
            cout<<'\n';
        }
    }
}

void solveHorizontal(int a){
    Node* bst = 0;
    int sz1 = n-a+1;
    int newNum;

    for(int i=0; i<n; i++){
        for(int j=0; j<a-1; j++)
            bst = insertNode(bst, data[i][j]);
        for(int j=a-1, k=0; j<n; j++, k++){
            bst = insertNode(bst, data[i][j]);
            newNum = getMax(bst);
            bst = removeNode(bst, data[i][k]);
            data[i][k] = newNum;
        }
        eraseTree(bst);
        bst = 0;
    }
    if(DEBUG){
        for(int i=0; i<n; i++){
            for(int j=0; j<sz1; j++){
                cout<<data[i][j]<<' ';
            }
            cout<<'\n';
        }
        cout<<'\n';
    }
}

void solveVerticalStupid(int sz, int b){
    int gSize=b;
    for(int j=0; j<sz; j++){
        int g=0;
        for(int i=0; i<b-1; i++){ //insert b-1 from row j
            precount[g] = data[i][j];
            g++;
        }
        for(int i=b-1, k=0; i<n; i++, k++){
            precount[g] = data[i][j];
            g = (g+1) % gSize;
            int maxx = 0;
            for(int l=0; l<gSize; l++)
                maxx=max(maxx, precount[l]);
            data[k][j] = maxx;
        }
    }
    if(DEBUG){
        for(int i=0; i<n-b+1; i++){
            for(int j=0; j<sz; j++){
                cout<<data[i][j]<<' ';
            }
            cout<<'\n';
        }
    }
}

void solveHorizontalStupid(int a){
    int gSize=a;

    for(int i=0; i<n; i++){
        int g=0;
        for(int j=0; j<a-1; j++){
            precount[g] = data[i][j];
            g++;
        }
        for(int j=a-1, k=0; j<n; j++, k++){
            precount[g] = data[i][j];
            g = (g+1) % gSize;
            int maxx = 0;
            for(int l=0; l<gSize; l++)
                maxx=max(maxx, precount[l]);
            data[i][k] = maxx;
        }
    }
    if(DEBUG){
        for(int i=0; i<n; i++){
            for(int j=0; j<n-a+1; j++){
                cout<<data[i][j]<<' ';
            }
            cout<<'\n';
        }
        cout<<'\n';
    }
}

int a, b;
int main(){
    srand(time(0));
    input();
    long shit = n*n*(a+b);
    if(shit > 2281488){
        solveHorizontal(a);
        solveVertical(n-a+1, b);
        int ans1 = getAnswer(n-b+1, n-a+1);
        getDataBack();
        solveHorizontal(b);
        solveVertical(n-b+1, a);
        cout<<min(ans1, getAnswer(n-a+1, n-b+1));
    } else {
        solveHorizontalStupid(a);
        solveVerticalStupid(n-a+1, b);
        int ans1 = getAnswer(n-b+1, n-a+1);
        getDataBack();
        solveHorizontalStupid(b);
        solveVerticalStupid(n-b+1, a);
        cout<<min(ans1, getAnswer(n-a+1, n-b+1));
    }
}

void input(){
    //ifstream cin("friendsTest3.txt", ifstream::in);
    ios::sync_with_stdio(0);
    cin.tie(0);
    cin>>n;
    cin>>a>>b;
    int input;
    for(int i=0; i<n; i++){
        for(int j=0; j<n; j++){
            cin>>input;
            data[i][j] = input;
            dataCP[i][j] = input;
        }
    }
}

void getDataBack(){
    for(int i=0; i<n; i++)
        for(int j=0; j<n; j++)
            data[i][j] = dataCP[i][j];
}
